"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.HttpStatus = void 0;
const uuid_base58_1 = require("uuid-base58");
var HttpStatus;
(function (HttpStatus) {
    HttpStatus[HttpStatus["OK"] = 200] = "OK";
    HttpStatus[HttpStatus["Created"] = 201] = "Created";
    HttpStatus[HttpStatus["Accepted"] = 202] = "Accepted";
    HttpStatus[HttpStatus["NoContent"] = 204] = "NoContent";
    HttpStatus[HttpStatus["BadRequest"] = 400] = "BadRequest";
    HttpStatus[HttpStatus["Unauthorized"] = 401] = "Unauthorized";
    HttpStatus[HttpStatus["Forbidden"] = 403] = "Forbidden";
    HttpStatus[HttpStatus["NotFound"] = 404] = "NotFound";
    HttpStatus[HttpStatus["MethodNotAllowed"] = 405] = "MethodNotAllowed";
    HttpStatus[HttpStatus["Conflict"] = 409] = "Conflict";
    HttpStatus[HttpStatus["InternalServerError"] = 500] = "InternalServerError";
    HttpStatus[HttpStatus["NotImplemented"] = 501] = "NotImplemented";
    HttpStatus[HttpStatus["BadGateway"] = 502] = "BadGateway";
    HttpStatus[HttpStatus["ServiceUnavailable"] = 503] = "ServiceUnavailable";
    HttpStatus[HttpStatus["GatewayTimeout"] = 504] = "GatewayTimeout";
})(HttpStatus = exports.HttpStatus || (exports.HttpStatus = {}));
class LambdaApp {
    constructor() {
        this._options = null;
        this._log = null;
        this._context = {
            awsRequestId: '',
            requestId: '',
            userAgent: '',
            sourceIp: ''
        };
        this._identity = {};
        this._error = null;
        this._startTime = new Date().getTime();
    }
    setRequestId(requestId) {
        this._context.requestId = requestId;
    }
    get error() {
        return this._error;
    }
    set error(e) {
        this._error = e;
    }
    getDefaultInitOptions() {
        return {
            authorize: true,
            accessControlAllowOrigin: '*'
        };
    }
    get log() {
        return (this._log || {
            info(message) {
                console.info(message);
            },
            error(message) {
                console.error(message);
            },
            debug(message) {
                console.debug(message);
            },
            warn(message) {
                console.warn(message);
            }
        });
    }
    set log(l) {
        this._log = l;
    }
    getInitialAppContext(event, context) {
        const awsRequestId = (0, uuid_base58_1.encode)(context.awsRequestId);
        const requestId = `${awsRequestId}`;
        return {
            awsRequestId,
            requestId,
            userAgent: event.requestContext?.http?.userAgent ?? '',
            sourceIp: event.requestContext?.http?.sourceIp ?? ''
        };
    }
    get options() {
        return this._options || this.getDefaultInitOptions();
    }
    /**
     * example context type might be APIGatewayProxyEventV2WithJWTAuthorizer
     * @returns
     */
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    async onLoadIdentity(requestContext) {
        return {};
    }
    get identity() {
        return this._identity;
    }
    set identity(i) {
        this._identity = i;
    }
    get context() {
        return this._context;
    }
    async init({ event, context, options }) {
        this._options = { ...this.getDefaultInitOptions(), ...options };
        this._context = this.getInitialAppContext(event, context);
        if (this.options.authorize) {
            return await this.onLoadIdentity(event.requestContext)
                .then((identity) => {
                this._identity = identity;
                return this;
            })
                .catch((error) => {
                this.error = error;
                return this;
            });
        }
        return Promise.resolve(this);
    }
    jsonResponse(httpStatus, json) {
        const responseHeaders = {
            'Content-Type': 'application/json',
            'Access-Control-Allow-Origin': this.options.accessControlAllowOrigin ?? '*'
        };
        return {
            statusCode: httpStatus,
            body: JSON.stringify(json),
            headers: responseHeaders
        };
    }
    get duration() {
        return new Date().getTime() - this._startTime;
    }
    response(httpStatus, json) {
        // if (this._options.writeRequestLog) {
        //   Logger.response(this._context, httpStatus)
        // }
        // remove context if empty
        return this.jsonResponse(httpStatus, {
            request: {
                id: this.context.requestId,
                duration: this.duration
            },
            ...{ context: Object.keys(this.identity).length === 0 ? undefined : this.identity },
            ...json
        });
    }
}
exports.default = LambdaApp;
