import { APIGatewayEventRequestContextV2, APIGatewayProxyEventV2WithRequestContext, APIGatewayProxyStructuredResultV2 } from 'aws-lambda/trigger/api-gateway-proxy';
import { Context } from 'aws-lambda/handler';
export declare enum HttpStatus {
    OK = 200,
    Created = 201,
    Accepted = 202,
    NoContent = 204,
    BadRequest = 400,
    Unauthorized = 401,
    Forbidden = 403,
    NotFound = 404,
    MethodNotAllowed = 405,
    Conflict = 409,
    InternalServerError = 500,
    NotImplemented = 501,
    BadGateway = 502,
    ServiceUnavailable = 503,
    GatewayTimeout = 504
}
export interface AppLoggerInterface {
    info: (message: string) => void;
    error: (message: string) => void;
    debug: (message: string) => void;
    warn: (message: string) => void;
}
export interface ApplicationInitOptions {
    /**
     * authorize the user when they init, DEFAULT is true
     */
    authorize?: boolean;
    accessControlAllowOrigin?: string;
}
export interface ApplicationInitRequest<TEvent> {
    event: TEvent;
    context: Context;
    options?: ApplicationInitOptions;
}
export interface AppContext {
    awsRequestId: string;
    requestId: string;
    userAgent: string;
    sourceIp: string;
}
export default class LambdaApp<AppIdentity, TRequestContext extends APIGatewayEventRequestContextV2, TEvent extends APIGatewayProxyEventV2WithRequestContext<TRequestContext>> {
    private _options;
    private _log;
    private _context;
    private _identity;
    private _error;
    private _startTime;
    setRequestId(requestId: string): void;
    get error(): Error | null;
    protected set error(e: Error | null);
    private getDefaultInitOptions;
    get log(): AppLoggerInterface;
    set log(l: AppLoggerInterface);
    private getInitialAppContext;
    get options(): ApplicationInitOptions;
    /**
     * example context type might be APIGatewayProxyEventV2WithJWTAuthorizer
     * @returns
     */
    onLoadIdentity(requestContext: TRequestContext): Promise<AppIdentity>;
    get identity(): AppIdentity;
    set identity(i: AppIdentity);
    get context(): AppContext;
    init({ event, context, options }: ApplicationInitRequest<TEvent>): Promise<LambdaApp<AppIdentity, TRequestContext, TEvent>>;
    private jsonResponse;
    get duration(): number;
    response(httpStatus: HttpStatus, json?: Record<string, any>): APIGatewayProxyStructuredResultV2;
}
